"""Provides functions related to Html page."""

from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

# CONSTANTS
WAIT_TIME = 3

class Page():
    """Page class"""
    def __init__(self, driver):
        """Constructor of Page class"""
        self.driver = driver

    def click_with_key_down(self, element, key):
        """Click the element while pressing the specified button.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        :rtype: string key
        """
        actions = ActionChains(self.driver)
        actions.key_down(key).click(element).key_up(key).perform()

    def scroll_to_element(self, id_or_class):
        """Scroll to the element specified by the selector.

        :rtype: string id_or_class
        """
        element = self.get_element_by_id(id_or_class)
        if element is None:
            element = self.get_element_by_class_name(id_or_class)
        if element is None:
            print('Cannot scroll to [' + id_or_class + '].')
        else:
            actions = ActionChains(self.driver)
            actions.move_to_element(element).perform()
        return element

    def activate_tab(self, index=-1):
        """Activate the window tab, which is specified by index.

        If index is not specified, activate the last tab.

        :rtype: number index
        """
        target_tab = self.driver.window_handles[index]
        self.driver.switch_to.window(target_tab)

    def close_inactive_tabs(self):
        """Close all the window tabs, which are not active."""
        while len(self.driver.window_handles) > 1:
            self.activate_tab(1)
            self.driver.close()
            self.activate_tab()

    def force_click(self, element):
        """Click the element via JavaScript.

        It is possible to click an element even when element.is_displayed() is False.
        """
        self.driver.execute_script('$(arguments[0]).click();', element)

    def get_element(self, selected_by, value):
        """Get the element by the specified selector.

        Wait until the element is fully loaded.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        """
        try:
            is_located = EC.presence_of_element_located((selected_by, value))
            return WebDriverWait(self.driver, WAIT_TIME).until(is_located)
        except TimeoutException:
            return None

    def get_elements(self, selected_by, value):
        """Get all the elements by the specified selector.

        Wait until the elements are fully loaded.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        """
        try:
            is_located = EC.presence_of_all_elements_located((selected_by, value))
            return WebDriverWait(self.driver, WAIT_TIME).until(is_located)
        except TimeoutException:
            return None

    def get_element_by_id(self, element_id):
        """Get the element specified by id.

        Wait until the element is fully loaded.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        """
        return self.get_element(By.ID, element_id)

    def get_element_by_class_name(self, class_name):
        """Get the element specified by class name.

        Wait until the element is fully loaded.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        """
        return self.get_element(By.CLASS_NAME, class_name)

    def get_elements_by_class_name(self, class_name):
        """Get all the elements specified by class name.

        Wait until the elements are fully loaded.

        :rtype: selenium.webdriver.remote.webelement.WebElement element
        """
        return self.get_elements(By.CLASS_NAME, class_name)



"""Create Chrome driver for Selenium"""

from selenium import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

class DriverFactory():
    """Class for creating Chrome driver for Selenium"""
    def __init__(self):
        """Constructor for DockerDriverFactory"""
        self.cmd_executor = 'http://selenium-hub:4444/wd/hub'
        self.capabilities = DesiredCapabilities.CHROME

    def set_capabilities(self, capabilities):
        """Set browser capabilities"""
        self.capabilities = capabilities

    def create(self):
        """Create Chrome driver"""
        return webdriver.Remote(
            command_executor=self.cmd_executor,
            desired_capabilities=self.capabilities)



"""Base Test Case"""

import os
import unittest
from selenium_unittest_common.common import driver_factory
from selenium_unittest_common.common import page as html_page

# CONSTANTS
DRIVER_FACTORY = driver_factory.DriverFactory()

class BaseTestCase(unittest.TestCase):
    """Base test case class"""
    @classmethod
    def setUpClass(cls):
        """Initialize browser driver and page modules."""
        super(BaseTestCase, cls).setUpClass()
        cls.driver = DRIVER_FACTORY.create()
        cls.html_page = html_page.Page(cls.driver)

    def tearDown(self):
        """Cleanup operation after each test is executed."""
        self.html_page.close_inactive_tabs()

        # Save a screenshot when error occur
        if hasattr(self, '_outcome'):
            result = self.defaultTestResult()
            self._feedErrorsToResult(result, self._outcome.errors)
            error = self.check_result_flag(result.errors)
            failure = self.check_result_flag(result.failures)
            if error or failure:
                self.save_screenshot()

        super(BaseTestCase, self).tearDown()

    def check_result_flag(self, exc_list):
        """Check if the specified flag is on

        :rtype: array exc_list
        """
        if exc_list and exc_list[-1][0] is self:
            return exc_list[-1][1]
        return False

    def shortDescription(self):
        """Do not show function comment upon running test cases"""
        return None

    def save_screenshot(self):
        """Save screenshot"""
        img_dir = 'screenshots'
        if not os.path.exists(img_dir):
            os.mkdir(img_dir)

        img_file = '{}/{}.png'.format(img_dir, self._testMethodName)
        self.driver.save_screenshot(img_file)



"""Create configuration"""

import configparser

class ConfigFactory():
    """ConfigFactory Class"""
    def __init__(self, config_file):
        """Constructor for ConfigFactory class"""
        self.config_file = config_file

    def create(self):
        """Read the configuration file and create a configuration object"""
        config = configparser.RawConfigParser(allow_no_value=True)
        config.read_file(open(self.config_file))
        return config



"""Send credentials to Basic Authentication."""

class BasicAuth():
    """BasicAuth Class"""
    def __init__(self, driver, hosts, is_ssl=True):
        """Constructor for BasicAuth class

        :rtype: selenium.webdriver driver
        :rtype: array hosts
        :rtype: boolean is_ssl
        """
        self.driver = driver
        self.hosts = hosts
        self.is_ssl = is_ssl
        self.login = ''
        self.pwd = ''

    def send_credentials(self, login, pwd):
        """Send credentials, which is as part of URL, to the host.

        :rtype: string login
        :rtype: string pwd
        """
        protocol = 'https://' if self.is_ssl else 'http://'
        for host in self.hosts:
            self.driver.get(protocol + login + ':' + pwd + '@' + host + '/')
